{

NWInfo.dpr

This file contains the source/project file for NWInfo.DLL


The main purpose of this DLL is to allow an existing application to check if the NetWare Client32 is installed on the current workstation, and - if it is - retrieve some data about the user, the drives, the servers, and the NDS environment.

Since this DLL tries to dynamically load the Client32 DLLs where necessary, it allows this check also on stations without the NetWare Client32, without bringing up a load error.

The usage for this DLL is mainly to smoothly integrate existing applications into NDS without making them too dependent on the NetWare Client:
The application can dynamically check the existence of the NetWare Client and NDS, and then take appropriate action.

A typical sample scenario would be the implementation of a "Single Login":

The application might be written to run on NetWare 3.x, 4.x, 5.x servers, but also on WinNT, or standalone.
Currently it might use a custom login/authentication screen to identify the user.

With this DLL the application could check, if NetWare/NDS is available, and retrieve the name/context/server or the user.
An additional, application specific authentication might no longer be required, and a "Single Login" is easily implemented.


This DLL can be called from any application that allows calling into 32bit DLLs


For illustration and test purposes a Delphi sample (NWTEST) is included, but this DLL is independent of that test case. 



Exported DLL functions:

Only two functions are exported by this DLL:


________________________________________________________________________________

NWInitialize

Check if the NetWare Client32 is installed on the current system

Syntax
     N_EXTERN_LIBRARY  NWCCODE  NWInitialize(void);

Pascal Syntax
     Function NWInitialize: NWCCODE;

Return Values
     These are common return values; see the NDK topic "Return Values" for more information.
        0       : success, Client is loaded
        -1      : Unable to Obtain Double Byte Information
        0x88FF  : SHELL_FAILURE
________________________________________________________________________________

NWGetInfo

Return several key informations about the current user/server/drive/context

Syntax
     N_EXTERN_LIBRARY  NWCCODE  NWGetInfo(nuint8  Drv,
                                          nuint8  InfoType,
                                          pnuint8 Info);

Pascal Syntax
     Function NWGetInfo(Drv: Byte; InfoType: Byte; Info: Pointer): NWCCODE;


Parameters
     Drv
         (IN) Specifies the drive number for which to get the status (A=1, B=2, C=3, . . .); pass 0 for current drive.
     InfoType
         (IN) Specifies the type of information that is requested.
              The current version handles these information requests:

              _value___Information________________________________type_______Example___________________________
                  00:  NetWare                                    pnstr8     volume:dir1\dir2
                  01:  Server Volume                              pnstr8     server\volume:dir1\dir2
                  02:  Drive                                      pnstr8     Q:\dir1\dir2
                  03:  UNC                                        pnstr8     \\server\volume\dir1\dir2
                  04:  UNC root                                   pnstr8     \\server\volume\dir1
                  05:  UNC relative                               pnstr8     \dir2
                  06:  drive status                               nuint      see NDK docs
                  07:  conn handle                                nuint      see NDK docs
                  11:  Authentication state                       nuint      (0=Not authenticated; 1=Bindery authentication; 2=NDS authentication)
                  12:  Broadcast state                            nuint      see NDK docs
                  13:  Connection reference                       nuint32    see NDK docs
                  14:  NDS tree name                              pnstr8     NOVELL_INC_____________________
                  15:  Connection number                          nuint      17
                  16:  User ID                                    nuint32    7600EB04
                  17:  Server Name                                pnstr8     FS4
                  18:  NDS State                                  nuint      (0=NWCC_NDS_NOT_CAPABLE; 1=NWCC_NDS_CAPABLE)
                  19:  NMax Packet Size                           nuint
                  1A:  License State                              nuint      (0=NWCC_NOT_LICENSED; 1=NWCC_CONNECTION_LICENSED; 2=NWCC_HANDLE_LICENSED)
                  1B:  Distance in Hubs                           nuint
                  1C:  Server Version                             NWCCVersion
                  21:  Name Context                               pnstr8     dept.org
                  22:  Tree Name                                  pnstr8     NOVELL_INC
                  23:  User Name (canonicalized, typeless)        pnstr8     WSchreib
                  24:  User Name (canonicalized, typeful)         pnstr8     CN=WSchreib
                  25:  User Name (typeless)                       pnstr8     WSchreib.dept.org
                  26:  User Name (typeful)                        pnstr8     CN=WSchreib.OU=Dept.O=Org
                  27:  NDS Server Name (canonicalized, typeless)  pnstr8     FS4
                  28:  NDS Server Name (canonicalized, typeful)   pnstr8     CN=FS4
                  29:  NDS Server Name (typeless)                 pnstr8     FS4.dept.org
                  2A:  NDS Server Name (typeful)                  pnstr8     CN=FS4.OU=Dept.O=Org

             for details on the returned information and error codes see the NetWare NDK documentation under ...
                 x00..x07: NWGetDriveStatus
                 x11..x1D: NWCCGetConnInfo
                 x21..x22: NWDSGetContext
                 x23..x26: NWDSMapIDToName
                 x27..x2A: NWDSGetServerDN

}

library NWInfo;

uses
  SysUtils, Classes, Windows;


{$R *.RES}

{$I NWTypes.inc}    // NetWare typedefs


// ------------------------------------------------------------------------------------------------------------


Function NWInitialize: NWCCODE;
export;  {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   DLLHandle    : THandle;
   xNWCallsInit : function(p1, p2: nptr): NWCCODE;        {$IFDEF Win32} stdcall; {$ENDIF}
begin
   result := $FFFF;
   DLLHandle := LoadLibrary(CALWIN);
{$IFDEF Win32}
   if DLLHandle<>0 then begin
{$Else}
   if DLLHandle>32 then begin
{$ENDIF}
      @xNWCallsInit := GetProcAddress(DLLHandle, 'NWCallsInit');
      if @xNWCallsInit<>nil then result := xNWCallsInit(nil, nil);
      FreeLibrary(DLLHandle);
      end;
end;

(*   for generic Win functions, see also

WNetGetConnection(
    LPCTSTR lpLocalName,	// pointer to local name
    LPTSTR  lpRemoteName,	// pointer to buffer for remote name
    LPDWORD lpnLength 	        // pointer to buffer size, in characters
   );

WNetGetUser(
    LPTSTR  lpszLocalName,	// address of local name to get user name for
    LPTSTR  lpszUserName,	// address of buffer for user name
    LPDWORD  lpcchBuffer 	// address of buffer size variable
   );
*)


Function
   NWInfo_NWCCGetConnInfo(connHandle: NWCONN_HANDLE;
                infoType : nuint;
                len      : nuint;
                buffer   : nptr
                )        : NWCCODE;                       {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWCCGetConnInfo : function(connHandle: NWCONN_HANDLE;
                infoType : nuint;
                len      : nuint;
                buffer   : nptr
                )        : NWRCODE;                       {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCLXInit : function(p1, p2: nptr): NWCCODE;          {$IFDEF Win32} stdcall; {$ENDIF}
   DLL : THandle;
begin
   // implement the function NWCCGetConnInfo
   result := $FFFF;
   DLL := LoadLibrary(CLXWIN);
   if DLL<>0 then begin
      @xNWCLXInit := GetProcAddress(DLL, 'NWCLXInit');
      if @xNWCLXInit<>nil then result := NWCCODE(xNWCLXInit(nil, nil));
      if result<>0 then exit;

      result := $FFFE;
      @xNWCCGetConnInfo := GetProcAddress(DLL, 'NWCCGetConnInfo');
      if @xNWCCGetConnInfo<>nil
         then result := NWCCODE(xNWCCGetConnInfo(connHandle, infoType, len, buffer));
      FreeLibrary(DLL);
      end;
end;


Function
   NWInfo_NWDSServer(connHandle: NWCONN_HANDLE) : nbool8; {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWCCGetConnInfo : function(connHandle: NWCONN_HANDLE;
                infoType : nuint;
                len      : nuint;
                buffer   : nptr
                )        : NWRCODE;                       {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCLXInit : function(p1, p2: nptr): NWCCODE;          {$IFDEF Win32} stdcall; {$ENDIF}
   NDS_State  : nuint;
   DLL        : THandle;
   cCode      : NWRCODE;
begin
   // implement the function NWCCGetConnInfo
   result := N_FALSE;
   DLL := LoadLibrary(CLXWIN);
   if DLL<>0 then begin
      @xNWCLXInit := GetProcAddress(DLL, 'NWCLXInit');
      if @xNWCLXInit<>nil then begin
         cCode := NWCCODE(xNWCLXInit(nil, nil));
         if cCode=0 then begin
            @xNWCCGetConnInfo := GetProcAddress(DLL, 'NWCCGetConnInfo');
            if @xNWCCGetConnInfo<>nil
               then cCode := NWCCODE(xNWCCGetConnInfo(connHandle, 8, sizeOf(NDS_State), @NDS_State));
            if (cCode=0) and (NDS_State = NWCC_NDS_CAPABLE) then result := N_TRUE;
            end;
         end;
      FreeLibrary(DLL);
      end;
end;


Function
   NWInfo_NWGetDriveStatus(driveNum: nuint16;
                pathFormat : nuint16;
                status     : pnuint16;
                Var conn   : NWCONN_HANDLE;
                rootPath, relPath, fullPath: pnstr8
                ) : NWCCODE;                              {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWGetDriveStatus : function(driveNum: nuint16 ;
                pathFormat : nuint16 ;
                status     : pnuint16 ;
                Var conn   : NWCONN_HANDLE;
                rootPath, relPath, fullPath: pnstr8
                ) : NWCCODE;                              {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCallsInit : function(p1, p2: nptr): NWCCODE;        {$IFDEF Win32} stdcall; {$ENDIF}
   DLL : THandle;
begin
   // implement the function NWGetDriveStatus
   result := $FFFF;
   DLL := LoadLibrary(CALWIN);
   if DLL<>0 then begin
      @xNWCallsInit := GetProcAddress(DLL, 'NWCallsInit');
      if @xNWCallsInit<>nil then result := xNWCallsInit(nil, nil);
      if result<>0 then exit;

      result := $FFFE;
      @xNWGetDriveStatus := GetProcAddress(DLL, 'NWGetDriveStatus');
      if @xNWGetDriveStatus<>nil
         then result := xNWGetDriveStatus(driveNum, pathFormat, status, conn, rootPath, relPath, fullPath);
      FreeLibrary(DLL);
      end;
end;


Function
   NWInfo_NWDSMapIDToName(conn : NWCONN_HANDLE; objectID : nuint32; flagCanonicalize, flagTypeless: Boolean;
               objectName : pnstr8) : NWDSCCODE;                                                           {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWDSCreateContextHandle : function(Var newHandle : NWDSContextHandle)                     : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSGetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSSetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSMapIDToName         : function(context : NWDSContextHandle; conn : NWCONN_HANDLE;
                                       objectID : nuint32; objectName : pnstr8)               : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSFreeContext         : function(context : NWDSContextHandle)                           : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWGetObjectName         : function(conn: NWCONN_HANDLE; objectID: nuint32;
                                       objectName: pnstr8; objType : pnuint16)                : NWCCODE;   {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCallsInit : function(p1, p2: nptr)                                                      : NWCCODE;   {$IFDEF Win32} stdcall; {$ENDIF}
   DLL1, DLL2   : THandle;
   context      : NWDSContextHandle;
   ctxFlags     : nuint32;
   objType      : nuint16;
CONST
   DCV_TYPELESS_NAMES      = $04;  // default: not set
   DCV_CANONICALIZE_NAMES  = $10;  // default: set
begin
   // implement the function NWDSMapIDToName (with context flag extension)
   result := $FFFF;
   DLL1 := LoadLibrary(CALWIN);
   if DLL1<>0 then begin
      @xNWCallsInit := GetProcAddress(DLL1, 'NWCallsInit');
      if (@xNWCallsInit = nil) then exit;

      result := xNWCallsInit(nil, nil);
      if result<>0 then exit;

      if (NWInfo_NWDSServer(conn) = N_TRUE)
         then begin  // try to get the NDS name
              DLL2 := LoadLibrary(NETWIN);
              if DLL2<>0 then begin
                 @xNWDSCreateContextHandle := GetProcAddress(DLL2, 'NWDSCreateContextHandle');
                 @xNWDSGetContext          := GetProcAddress(DLL2, 'NWDSGetContext');
                 @xNWDSSetContext          := GetProcAddress(DLL2, 'NWDSSetContext');
                 @xNWDSFreeContext         := GetProcAddress(DLL2, 'NWDSFreeContext');
                 @xNWDSMapIDToName         := GetProcAddress(DLL2, 'NWDSMapIDToName');
                 if (@xNWDSCreateContextHandle = nil) or
                    (@xNWDSFreeContext         = nil) or
                    (@xNWDSGetContext          = nil) or
                    (@xNWDSSetContext          = nil) or
                    (@xNWDSMapIDToName         = nil) then result := $FFFE;
                 if result=0 then result := xNWDSCreateContextHandle(context);
                 if result=0 then result := xNWDSGetContext(context, 1, @ctxFlags);
                 if result=0 then begin
                    if flagTypeless          then ctxFlags := ctxFlags or DCV_TYPELESS_NAMES;
                    if not flagCanonicalize  then ctxFlags := ctxFlags and (not DCV_CANONICALIZE_NAMES);
                    result := xNWDSSetContext(context, 1, @ctxFlags);
                    end;
                 if result=0 then result := xNWDSMapIDToName(context, conn, objectID, objectName);
                 if result=0 then result := xNWDSFreeContext(context);
                 FreeLibrary(DLL2);
                 end
              end
         else begin  // try to get the bindery name
              @xNWGetObjectName := GetProcAddress(DLL1, 'NWGetObjectName');
              if (@xNWGetObjectName = nil) then result := $FFFE;
              if result=0 then result := xNWGetObjectName(conn, objectID, objectName, @objType);
              end;

      FreeLibrary(DLL1);
      end;
end;


Function
   NWInfo_NWDSGetServerDN(conn : NWCONN_HANDLE; flagCanonicalize, flagTypeless: Boolean;
               objectName : pnstr8) : NWDSCCODE;                                                           {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWDSCreateContextHandle : function(Var newHandle : NWDSContextHandle)                     : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSGetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSSetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSGetServerDN         : function(context : NWDSContextHandle; conn : NWCONN_HANDLE;
                                       objectName : pnstr8)                                   : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSFreeContext         : function(context : NWDSContextHandle)                           : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCallsInit : function(p1, p2: nptr)                                                      : NWCCODE;   {$IFDEF Win32} stdcall; {$ENDIF}
   DLL1, DLL2   : THandle;
   context      : NWDSContextHandle;
   ctxFlags     : nuint32;
CONST
   DCV_TYPELESS_NAMES      = $04;  // default: not set
   DCV_CANONICALIZE_NAMES  = $10;  // default: set
begin
   // implement the function NWDSGetServerDN (with context flag extension)
   result := $FFFF;
   DLL1 := LoadLibrary(CALWIN);
   if DLL1<>0 then begin
      @xNWCallsInit := GetProcAddress(DLL1, 'NWCallsInit');
      if (@xNWCallsInit = nil) then exit;

      result := xNWCallsInit(nil, nil);
      if result<>0 then exit;

      DLL2 := LoadLibrary(NETWIN);
      if DLL2<>0 then begin
         @xNWDSCreateContextHandle := GetProcAddress(DLL2, 'NWDSCreateContextHandle');
         @xNWDSGetContext          := GetProcAddress(DLL2, 'NWDSGetContext');
         @xNWDSSetContext          := GetProcAddress(DLL2, 'NWDSSetContext');
         @xNWDSFreeContext         := GetProcAddress(DLL2, 'NWDSFreeContext');
         @xNWDSGetServerDN         := GetProcAddress(DLL2, 'NWDSGetServerDN');
         if (@xNWDSCreateContextHandle = nil) or
            (@xNWDSFreeContext         = nil) or
            (@xNWDSGetContext          = nil) or
            (@xNWDSSetContext          = nil) or
            (@xNWDSGetServerDN         = nil) then result := $FFFE;
         if result=0 then result := xNWDSCreateContextHandle(context);
         if result=0 then result := xNWDSGetContext(context, 1, @ctxFlags);
         if result=0 then begin
            if flagTypeless          then ctxFlags := ctxFlags or DCV_TYPELESS_NAMES;
            if not flagCanonicalize  then ctxFlags := ctxFlags and (not DCV_CANONICALIZE_NAMES);
            result := xNWDSSetContext(context, 1, @ctxFlags);
            end;
         if result=0 then result := xNWDSGetServerDN(context, conn, objectName);
         if result=0 then result := xNWDSFreeContext(context);
         FreeLibrary(DLL2);
         end;
      FreeLibrary(DLL1);
      end;
end;


Function
   NWInfo_NWDSGetContextInfo(key : nint; data : nptr) : NWDSCCODE;                                         {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   xNWDSCreateContextHandle : function(Var newHandle : NWDSContextHandle)                     : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSFreeContext         : function(context : NWDSContextHandle)                           : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSGetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWDSSetContext          : function(context : NWDSContextHandle; key : nint; value : nptr) : NWDSCCODE; {$IFDEF Win32} stdcall; {$ENDIF}
   xNWCallsInit             : function(p1, p2: nptr): NWCCODE;                                             {$IFDEF Win32} stdcall; {$ENDIF}
   DLL1, DLL2   : THandle;
   context      : NWDSContextHandle;
begin
   result := $FFFF;
   DLL1 := LoadLibrary(CALWIN);
   if DLL1<>0 then begin
      @xNWCallsInit := GetProcAddress(DLL1, 'NWCallsInit');
      if (@xNWCallsInit = nil) then exit;

      result := xNWCallsInit(nil, nil);
      if result<>0 then exit;


      DLL2 := LoadLibrary(NETWIN);
      if DLL2<>0 then begin
         @xNWDSCreateContextHandle := GetProcAddress(DLL2, 'NWDSCreateContextHandle');
         @xNWDSFreeContext         := GetProcAddress(DLL2, 'NWDSFreeContext');
         @xNWDSGetContext          := GetProcAddress(DLL2, 'NWDSGetContext');
         @xNWDSSetContext          := GetProcAddress(DLL2, 'NWDSSetContext');

         if (@xNWDSCreateContextHandle = nil) or
            (@xNWDSFreeContext         = nil) or
            (@xNWDSGetContext          = nil) or
            (@xNWDSSetContext          = nil) then result := $FFFE;

         if result=0 then result := xNWDSCreateContextHandle(context);
         if result=0 then result := xNWDSGetContext(context, key, data);
         if result=0 then result := xNWDSFreeContext(context);

         FreeLibrary(DLL2);
         end;

      FreeLibrary(DLL1);
      end;
end;


Function
   NWInfo_WhoAmI(Drv: nuint16; objectName: pnstr8; flagCanonicalize, flagTypeless: Boolean) : NWDSCCODE;                             {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   conn             : NWCONN_HANDLE;
   userID           : nuint32;
   Paths            : Array[1..3] of TNWPath;
   status           : nuint16;
begin
   // first, get the conn for the drive (from CALWIN)
   result := NWInfo_NWGetDriveStatus(Drv, 0, @status, conn, @Paths[1], @Paths[2], @Paths[3]);

   // I've got my connection, now get the user ID (from CLXWIN)
   if result=0
      then result := NWInfo_NWCCGetConnInfo(conn, 6, sizeof(nuint32), @userID);

   // I've got my ID, now get the NDS name (from NETWIN)
   if result=0
      then result := NWInfo_NWDSMapIDToName(conn, userID, flagCanonicalize, flagTypeless, objectName);
end;


Function
   NWInfo_ServerName(Drv: nuint16; objectName: pnstr8; flagCanonicalize, flagTypeless: Boolean) : NWDSCCODE;                             {$IFDEF Win32} stdcall; {$ENDIF}
VAR
   conn             : NWCONN_HANDLE;
   userID           : nuint32;
   Paths            : Array[1..3] of TNWPath;
   status           : nuint16;
begin
   // first, get the conn for the drive (from CALWIN)
   result := NWInfo_NWGetDriveStatus(Drv, 0, @status, conn, @Paths[1], @Paths[2], @Paths[3]);

   // I've got my connection, now get the user ID (from CLXWIN)
   if result=0
      then result := NWInfo_NWCCGetConnInfo(conn, 6, sizeof(nuint32), @userID);

   // I've got my ID, now get the NDS name (from NETWIN) or the server name (from CLXWIN)
   if result=0 then
      if (NWInfo_NWDSServer(conn) = N_TRUE)   // try to get the NDS name
         then result := NWInfo_NWDSGetServerDN(conn, flagCanonicalize, flagTypeless, objectName)
         else result := NWInfo_NWCCGetConnInfo(conn, 7, MaxDataSize, objectName);
end;


Function NWGetInfo(Drv: Byte; InfoType: Byte; Info: Pointer): NWCCODE;
export;  {$IFDEF Win32} stdcall; {$ENDIF}
{  InfoType  specifies the desired information

             for details on the returned values see the NetWare NDK documentation under ...
                 x00..x07: NWGetDriveStatus
                 x11..x1D: NWCCGetConnInfo
                 x21..x22: NWDSGetContext
                 x23..x26: NWDSMapIDToName
                 x27..x2A: NWDSGetServerDN

             00:  NetWare        (volume:dir1\dir2)
             01:  Server Volume  (server\volume:dir1\dir2)
             02:  Drive          (Q:\dir1\dir2)
             03:  UNC            (\\server\volume\dir1\dir2)
             04:  UNC root       (\\server\volume\dir1)
             05:  UNC relative   (\dir2)
             06:  drive status   ()
             07:  conn handle    ()
             08:
             09:
             0A:
             0B:
             0C:
             0D:
             0E:
             0F:
             10:
             11:  NWCC_INFO_AUTHENT_STATE
             12:  NWCC_INFO_BCAST_STATE
             13:  NWCC_INFO_CONN_REF
             14:  NWCC_INFO_TREE_NAME
             15:  NWCC_INFO_CONN_NUMBER
             16:  NWCC_INFO_USER_ID
             17:  NWCC_INFO_SERVER_NAME
             18:  NWCC_INFO_NDS_STATE
             19:  NWCC_INFO_MAX_PACKET_SIZE
             1A:  NWCC_INFO_LICENSE_STATE
             1B:  NWCC_INFO_DISTANCE
             1C:  NWCC_INFO_SERVER_VERSION
             1D:  
             1E:
             1F:
             20:
             21:  Name Context
             22:  Tree Name

             23:  NWDS WhoAmI (canonicalized, typeless)
             24:  NWDS WhoAmI (canonicalized, typeful)
             25:  NWDS WhoAmI (typeless)
             26:  NWDS WhoAmI (typeful)

             27:  NWDS Server Name (canonicalized, typeless)
             28:  NWDS Server Name (canonicalized, typeful)
             29:  NWDS Server Name (typeless)
             2A:  NWDS Server Name (typeful)
}
VAR
   Paths  : Array[1..3] of TNWPath;
   status : nuint16;
   conn   : NWCONN_HANDLE;
begin
   result := $FFCD;
   case InfoType of
        0..3: result := NWInfo_NWGetDriveStatus(Drv, InfoType, @status, conn,                 @Paths[1], @Paths[2], Info);
        4:    result := NWInfo_NWGetDriveStatus(Drv, 3,        @status, conn,                 Info,      @Paths[2], @Paths[3]);
        5:    result := NWInfo_NWGetDriveStatus(Drv, 3,        @status, conn,                 @Paths[1], Info,      @Paths[3]);
        6:    result := NWInfo_NWGetDriveStatus(Drv, 0,        Info,    conn,                 @Paths[1], @Paths[2], @Paths[3]);
        7:    result := NWInfo_NWGetDriveStatus(Drv, 0,        @status, NWCONN_HANDLE(Info^), @Paths[1], @Paths[2], @Paths[3]);
        $11..$1C: begin
              // first get the connection handle
              result := NWInfo_NWGetDriveStatus(Drv, 0,        @status, conn,                 @Paths[1], @Paths[2], @Paths[3]);
              // now get details about this connection
              if (result=0) then result := NWInfo_NWCCGetConnInfo(conn, (InfoType and $0F), MaxDataSize, Info);
              end;
        $21:  result := NWInfo_NWDSGetContextInfo(3,  Info);
        $22:  result := NWInfo_NWDSGetContextInfo(11, Info);
        $23:  result := NWInfo_WhoAmI(Drv, Info, true,  true);
        $24:  result := NWInfo_WhoAmI(Drv, Info, true,  false);
        $25:  result := NWInfo_WhoAmI(Drv, Info, false, true);
        $26:  result := NWInfo_WhoAmI(Drv, Info, false, false);
        $27:  result := NWInfo_ServerName(Drv, Info, true,  true);
        $28:  result := NWInfo_ServerName(Drv, Info, true,  false);
        $29:  result := NWInfo_ServerName(Drv, Info, false, true);
        $2A:  result := NWInfo_ServerName(Drv, Info, false, false);

        end; {case}
end;


// ------------------------------------------------------------------------------------------------------------

exports

   NWInitialize     index 1 resident,
   NWGetInfo        index 2 resident;

begin
end.
